<?php

/**
 * EYEMAGINE - The leading Magento Solution Partner
 *
 * HubSpot Integration with Magento
 *
 * @author    EYEMAGINE <magento@eyemaginetech.com>
 * @copyright Copyright (c) 2016 EYEMAGINE Technology, LLC (http://www.eyemaginetech.com)
 * @license   http://www.eyemaginetech.com/license
 */
namespace Eyemagine\HubSpot\Helper;

use Eyemagine\HubSpot\Controller\SyncInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\ScopeInterface;
use Psr\Log\LoggerInterface;

/**
 * Class Sync
 *
 * @package Eyemagine\HubSpot\Helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper implements SyncInterface
{
    const XML_HUBSPOT_API_KEY = 'eyehubspot/settings/api_key';
    const HUBSPOT_UPDATE_CONTACT_URL = 'https://api.hubapi.com/contacts/v1/contact/vid/%d/profile?hapikey=';
    const HUBSPOT_GET_CONTACT_BY_EMAIL_URL = 'https://api.hubapi.com/contacts/v1/contact/email/%s/profile?hapikey=';
    const HUBSPOT_CREATE_GROUP_URL = 'https://api.hubapi.com/properties/v1/contacts/groups?hapikey=';
    const HUBSPOT_CREATE_PROPERTY_URL = 'https://api.hubapi.com/properties/v1/contacts/properties?hapikey=';

    const API_METHOD_POST = 'POST';
    const API_METHOD_GET = 'GET';

    /**
     * Logger
     *
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Json
     */
    protected $json;

    public function __construct(
        Context $context,
        ScopeConfigInterface $scopeConfig,
        Json $json,
        LoggerInterface $logger
    )
    {
        parent::__construct($context);
        $this->scopeConfig = $scopeConfig;
        $this->json = $json;
        $this->logger = $logger;
    }

    /**
     * Send data to the API endpoint
     *
     * @var string $apiUrl API endpoint
     * @var string $method POST|GET|DELETE|PUT
     * @var string $body API payload
     */
    public function postToApi($apiUrl, $method = 'POST', $body = null)
    {
        try {
            $apiUrl .= $this->getApiKey();
            $httpHeader = array(
                "accept: application/json",
                "Content-Type: application/json"
            );

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $apiUrl);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($body));
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $httpHeader);
            // Allowing cUrl funtions 20 second to execute
            curl_setopt($ch, CURLOPT_TIMEOUT, 20);
            // Waiting 20 seconds while trying to connect
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 20);

            $response = curl_exec($ch);
            $curlInfo = curl_getinfo($ch);
            curl_close($ch);

            return $response;
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
        }

        return null;
    }

    /**
     * Get hubspot API Key
     * @return mixed
     */
    protected function getApiKey()
    {
        return $this->scopeConfig->getValue(self::XML_HUBSPOT_API_KEY, ScopeInterface::SCOPE_STORE);
    }

    /**
     * @param $json
     * @return array|bool|float|int|mixed|string|null
     */
    public function convertJsonToArray($json)
    {
        return $this->json->unserialize($json);
    }

    /**
     * Update contact
     * @param $contactId
     * @param null $body
     * @return bool|string|null
     */
    public function updateContact($contactId, $body = null)
    {
        $url = sprintf(self::HUBSPOT_UPDATE_CONTACT_URL, $contactId);
        return $this->postToApi($url, self::API_METHOD_POST, $body);
    }

    /**
     * Get contact by email
     * @param $email
     * @return bool|string|null
     */
    public function getContactByEmail($email)
    {
        $url = sprintf(self::HUBSPOT_GET_CONTACT_BY_EMAIL_URL, $email);
        return $this->postToApi($url, self::API_METHOD_GET);
    }

    /**
     * Create a new group
     * @param null $body
     * @return bool|string|null
     */
    public function createGroup($body = null)
    {
        return $this->postToApi(self::HUBSPOT_CREATE_GROUP_URL, self::API_METHOD_POST, $body);
    }

    /**
     * @param null $body
     * @return bool|string|null
     */
    public function createContactProperty($body = null)
    {
        return $this->postToApi(self::HUBSPOT_CREATE_PROPERTY_URL, self::API_METHOD_POST, $body);
    }
}
